@@ -15,24 +15,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * The Totem project hereby grant permission for non-gpl compatible GStreamer
- * plugins to be used and distributed together with GStreamer and Totem. This
- * permission are above and beyond the permissions granted by the GPL license
- * Totem is covered by.
- *
- * Monday 7th February 2005: Christian Schaller: Add excemption clause.
- * See license_change file for details.
- * 
  */
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <errno.h>
 
 #include "bacon-message-connection.h"
 
@@ -41,10 +35,17 @@
 #endif
 
 struct BaconMessageConnection {
+	/* A server accepts connections */
 	gboolean is_server;
-	int fd;
+
+	/* The socket path itself */
 	char *path;
+
+	/* FD is for the connection, serverfd is for the server socket,
+	 * it accepts incoming connections. */
+	int fd, serverfd, server_conn_id, conn_id;
 	GIOChannel *chan;
+
 	/* callback */
 	void (*func) (const char *message, gpointer user_data);
 	gpointer data;
@@ -64,37 +65,65 @@ test_is_socket (const char *path)
 	return FALSE;
 }
 
-#define BUF_SIZE 1024
+static gboolean server_cb (GIOChannel *source,
+			   GIOCondition condition, gpointer data);
+
+static gboolean
+setup_connection (BaconMessageConnection *conn)
+{
+	conn->chan = g_io_channel_unix_new (conn->fd);
+	if (!conn->chan) {
+		return FALSE;
+	}
+	g_io_channel_set_line_term (conn->chan, "\n", 1);
+	conn->conn_id = g_io_add_watch (conn->chan, G_IO_IN, server_cb, conn);
+
+	return TRUE;
+}
 
 static gboolean
 server_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 {
 	BaconMessageConnection *conn = (BaconMessageConnection *)data;
-	char *message, *subs, buf[BUF_SIZE];
+	char *message, *subs, buf;
 	int cd, alen, rc, offset;
 	gboolean finished;
 
-	message = NULL;
+	g_print ("Server-cb\n");
 	offset = 0;
-	cd = accept (g_io_channel_unix_get_fd (source), NULL, &alen);
-
-	memset (buf, 0, sizeof (buf));
-	rc = read (cd, buf, BUF_SIZE);
-//FIXME test for the actual errno
-	while (rc != 0)
-	{
-		message = g_realloc (message, rc + offset);
-		memcpy (message + offset, buf, MIN(rc, BUF_SIZE));
-
-		offset = offset + rc;
-		memset (buf, 0, sizeof (buf));
-		rc = read (cd, buf, BUF_SIZE);
+	if (conn->serverfd == g_io_channel_unix_get_fd (source)) {
+		cd = accept (conn->serverfd, NULL, &alen);
+		conn->fd = cd;
+		setup_connection (conn);
+		return TRUE;
 	}
+	message = g_malloc (1);
+	cd = conn->fd;
+	rc = read (cd, &buf, 1);
+	while (rc > 0 && buf != '\n')
+	{
+		message = g_realloc (message, rc + offset + 1);
+		message[offset] = buf;
+		offset = offset + rc;
+		rc = read (cd, &buf, 1);
+	}
+	if (rc <= 0) {
+		g_io_channel_shutdown (conn->chan, FALSE, NULL);
+		g_io_channel_unref (conn->chan);
+		conn->chan = NULL;
+		close (conn->fd);
+		conn->fd = -1;
+		g_free (message);
+		conn->conn_id = 0;
+
+		return FALSE;
+	}
+	message[offset] = '\0';
 
 	subs = message;
 	finished = FALSE;
 
-	while (subs != '\0' && finished == FALSE)
+	while (*subs != '\0' && finished == FALSE)
 	{
 		if (message != NULL && conn->func != NULL)
 			(*conn->func) (subs, conn->data);
@@ -109,30 +138,45 @@ server_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 	return TRUE;
 }
 
+static char *
+socket_filename (const char *prefix)
+{
+	char *filename, *path;
+	const char *dir;
+
+	filename = g_strdup_printf (".%s.%s", prefix, g_get_user_name ());
+
+	dir = g_getenv ("BACON_SOCKET_DIR");
+	if (dir == NULL)
+	{
+		path = g_build_filename (g_get_home_dir (), filename, NULL);
+	} else {
+		path = g_build_filename (dir, filename, NULL);
+	}
+
+	g_free (filename);
+	return path;
+}
+
 static gboolean
 try_server (BaconMessageConnection *conn)
 {
 	struct sockaddr_un uaddr;
-	GError *err = NULL;
 
 	uaddr.sun_family = AF_UNIX;
 	strncpy (uaddr.sun_path, conn->path,
 			MIN (strlen(conn->path)+1, UNIX_PATH_MAX));
-	conn->fd = socket (PF_UNIX, SOCK_STREAM, 0);
+	conn->serverfd = conn->fd = socket (PF_UNIX, SOCK_STREAM, 0);
 	if (bind (conn->fd, (struct sockaddr *) &uaddr, sizeof (uaddr)) == -1)
 	{
 		conn->fd = -1;
 		return FALSE;
 	}
-
 	listen (conn->fd, 5);
-	conn->chan = g_io_channel_unix_new (conn->fd);
-	if (conn->chan == NULL || err != NULL)
-	{
-		conn->fd = -1;
-		return FALSE;
-	}
 
+	if (!setup_connection (conn))
+		return FALSE;
+	conn->server_conn_id = conn->conn_id;
 	return TRUE;
 }
 
@@ -145,6 +189,7 @@ try_client (BaconMessageConnection *conn)
 	strncpy (uaddr.sun_path, conn->path,
 			MIN(strlen(conn->path)+1, UNIX_PATH_MAX));
 	conn->fd = socket (PF_UNIX, SOCK_STREAM, 0);
+	conn->serverfd = -1;
 	if (connect (conn->fd, (struct sockaddr *) &uaddr,
 				sizeof (uaddr)) == -1)
 	{
@@ -152,28 +197,22 @@ try_client (BaconMessageConnection *conn)
 		return FALSE;
 	}
 
-	return TRUE;
+	return setup_connection (conn);
 }
 
 BaconMessageConnection *
 bacon_message_connection_new (const char *prefix)
 {
 	BaconMessageConnection *conn;
-	char *filename, *path;
 
 	g_return_val_if_fail (prefix != NULL, NULL);
 
-	filename = g_strdup_printf (".%s.%s", prefix, g_get_user_name ());
-	path = g_build_filename (g_get_home_dir (), filename, NULL);
-	g_free (filename);
-
 	conn = g_new0 (BaconMessageConnection, 1);
-	conn->path = path;
+	conn->path = socket_filename (prefix);
 
 	if (test_is_socket (conn->path) == FALSE)
 	{
-		try_server (conn);
-		if (conn->fd == -1)
+		if (!try_server (conn))
 		{
 			bacon_message_connection_free (conn);
 			return NULL;
@@ -185,7 +224,7 @@ bacon_message_connection_new (const char *prefix)
 
 	if (try_client (conn) == FALSE)
 	{
-		unlink (path);
+		unlink (conn->path);
 		try_server (conn);
 		if (conn->fd == -1)
 		{
@@ -207,12 +246,23 @@ bacon_message_connection_free (BaconMessageConnection *conn)
 	g_return_if_fail (conn != NULL);
 	g_return_if_fail (conn->path != NULL);
 
-	if (conn->is_server != FALSE)
-	{
+	if (conn->server_conn_id) {
+		g_source_remove (conn->server_conn_id);
+		conn->server_conn_id = 0;
+	}
+	if (conn->conn_id) {
+		g_source_remove (conn->conn_id);
+		conn->conn_id = 0;
+	}
+	if (conn->chan) {
 		g_io_channel_shutdown (conn->chan, FALSE, NULL);
 		g_io_channel_unref (conn->chan);
+	}
+
+	if (conn->is_server != FALSE) {
 		unlink (conn->path);
-	} else {
+		close (conn->serverfd);
+	} else if (conn->fd != -1) {
 		close (conn->fd);
 	}
 
@@ -226,9 +276,6 @@ bacon_message_connection_set_callback (BaconMessageConnection *conn,
 				       gpointer user_data)
 {
 	g_return_if_fail (conn != NULL);
-	g_assert (conn->is_server != FALSE);
-
-	g_io_add_watch (conn->chan, G_IO_IN, server_cb, conn);
 
 	conn->func = func;
 	conn->data = user_data;
@@ -239,15 +286,18 @@ bacon_message_connection_send (BaconMessageConnection *conn,
 			       const char *message)
 {
 	g_return_if_fail (conn != NULL);
-	g_assert (conn->is_server == FALSE);
-//FIXME test short writes
-	write (conn->fd, message, strlen (message) + 1);
+
+	g_io_channel_write_chars (conn->chan, message, strlen (message),
+				  NULL, NULL);
+	g_io_channel_write_chars (conn->chan, "\n", 1, NULL, NULL);
+	g_io_channel_flush (conn->chan, NULL);
 }
 
 gboolean
 bacon_message_connection_get_is_server (BaconMessageConnection *conn)
 {
 	g_return_val_if_fail (conn != NULL, FALSE);
+
 	return conn->is_server;
 }
 
