@@ -394,7 +394,7 @@ drop_cb (GtkWidget     *widget,
 				}
 			}
 
-			totem_playlist_add_mrl (playlist, filename, title);
+			totem_playlist_add_mrl (playlist, p->data, title);
 		}
 		g_free (filename);
 		g_free (p->data);
@@ -536,7 +536,7 @@ button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 		totem_playlist_set_reorderable (playlist, FALSE);
 		gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 				target_table, G_N_ELEMENTS (target_table),
-				GDK_ACTION_COPY);
+				GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 		g_signal_handlers_unblock_by_func (treeview,
 				(GFunc) drop_cb, data);
@@ -563,7 +563,8 @@ drag_end_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 	playlist->_priv->drag_started = FALSE;
 	totem_playlist_set_reorderable (playlist, FALSE);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, target_table,
-        		G_N_ELEMENTS (target_table), GDK_ACTION_COPY);
+        		G_N_ELEMENTS (target_table),
+			GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	g_signal_handlers_unblock_by_func (treeview, (GFunc) drop_cb, data);
 
@@ -1148,7 +1149,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
                         G_CALLBACK (drag_end_cb), playlist);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY);
+			GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	playlist->_priv->selection = selection;
 
@@ -1605,8 +1606,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 				filename_for_display, uri, display_name);
 
 	store = GTK_LIST_STORE (playlist->_priv->model);
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter,
+	gtk_list_store_insert_with_values (store, &iter, G_MAXINT32,
 			PIX_COL, NULL,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri,
@@ -1956,7 +1956,6 @@ void
 totem_playlist_set_previous (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
-	char *path;
 
 	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
 
@@ -1965,18 +1964,19 @@ totem_playlist_set_previous (TotemPlaylist *playlist)
 
 	totem_playlist_unset_playing (playlist);
 
-	path = gtk_tree_path_to_string (playlist->_priv->current);
-	if (strcmp (path, "0") == 0)
-	{
-		totem_playlist_set_at_end (playlist);
-		g_free (path);
-		return;
-	}
-
-	g_free (path);
-
 	if (playlist->_priv->shuffle == FALSE)
 	{
+		char *path;
+
+		path = gtk_tree_path_to_string (playlist->_priv->current);
+		if (strcmp (path, "0") == 0)
+		{
+			totem_playlist_set_at_end (playlist);
+			g_free (path);
+			return;
+		}
+		g_free (path);
+
 		gtk_tree_model_get_iter (playlist->_priv->model,
 				&iter,
 				playlist->_priv->current);
@@ -1991,9 +1991,12 @@ totem_playlist_set_previous (TotemPlaylist *playlist)
 
 		gtk_tree_path_free (playlist->_priv->current);
 		playlist->_priv->current_shuffled--;
-		if (playlist->_priv->current_shuffled == 0)
+		if (playlist->_priv->current_shuffled < 0) {
 			indice = playlist->_priv->shuffled[PL_LEN -1];
-		indice = playlist->_priv->shuffled[playlist->_priv->current_shuffled];
+			playlist->_priv->current_shuffled = PL_LEN -1;
+		} else {
+			indice = playlist->_priv->shuffled[playlist->_priv->current_shuffled];
+		}
 		playlist->_priv->current = gtk_tree_path_new_from_indices
 			(indice, -1);
 	}
