@@ -40,6 +40,7 @@ void	totem_action_exit			(Totem *totem);
 void	totem_action_play			(Totem *totem);
 void	totem_action_stop			(Totem *totem);
 void	totem_action_play_pause			(Totem *totem);
+void	totem_action_pause			(Totem *totem);
 void	totem_action_fullscreen_toggle		(Totem *totem);
 void	totem_action_fullscreen			(Totem *totem, gboolean state);
 void	totem_action_next			(Totem *totem);
@@ -48,6 +49,9 @@ void	totem_action_seek_relative		(Totem *totem, int off_sec);
 void	totem_action_volume_relative		(Totem *totem, int off_pct);
 gboolean totem_action_set_mrl			(Totem *totem,
 						 const char *mrl);
+void	totem_action_set_mrl_and_play		(Totem *totem,
+						 const char *mrl);
+
 gboolean totem_action_set_mrl_with_warning	(Totem *totem,
 						 const char *mrl,
 						 gboolean warn);
